/**************************************************************************
*  Copyright (c) 2013 by Michael Fischer. All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  17.02.2013 mifi  First Version for the BeageBone board.
*                   CrossWorks for ARM v2.3.0 was used.
**************************************************************************/

/*=======================================================================*/
/*  Include                                                              */
/*=======================================================================*/
#include "ch.h"
#include "hal.h"

#include "mmu.h"
#include "cache.h"
#include "interrupt.h"

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

/*
 * Some defines for the MMU translation table
 */
#define START_ADDR_DDR      (0x80000000)
#define START_ADDR_DEV      (0x44000000)
#define START_ADDR_OCMC     (0x40300000)
#define NUM_SECTIONS_DDR    (512)
#define NUM_SECTIONS_DEV    (960)
#define NUM_SECTIONS_OCMC   (1)

/*=======================================================================*/
/*  Prototypes                                                           */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

static volatile unsigned int pageTable[4*1024] __attribute__((aligned(16*1024)));

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/
/*************************************************************************/
/*  MMUConfigAndEnable                                                   */
/*                                                                       */
/*  Function to setup MMU. This function Maps three regions              */
/*  1. DDR, 2. OCMC and 3. Device memory, and enables the MMU.           */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void MMUConfigAndEnable (void)
{
  /*
  ** Define DDR memory region of AM335x. DDR can be configured as Normal
  ** memory with R/W access in user/privileged modes. The cache attributes
  ** specified here are,
  ** Inner - Write through, No Write Allocate
  ** Outer - Write Back, Write Allocate
  */
  REGION regionDdr = {
                      MMU_PGTYPE_SECTION, START_ADDR_DDR, NUM_SECTIONS_DDR,
                      MMU_MEMTYPE_NORMAL_NON_SHAREABLE(MMU_CACHE_WT_NOWA,
                                                       MMU_CACHE_WB_WA),
                      MMU_REGION_NON_SECURE, MMU_AP_PRV_RW_USR_RW,
                      (unsigned int*)pageTable
                     };
  /*
  ** Define OCMC RAM region of AM335x. Same Attributes of DDR region given.
  */
  REGION regionOcmc = {
                       MMU_PGTYPE_SECTION, START_ADDR_OCMC, NUM_SECTIONS_OCMC,
                       MMU_MEMTYPE_NORMAL_NON_SHAREABLE(MMU_CACHE_WT_NOWA,
                                                        MMU_CACHE_WB_WA),
                       MMU_REGION_NON_SECURE, MMU_AP_PRV_RW_USR_RW,
                       (unsigned int*)pageTable
                      };

  /*
  ** Define Device Memory Region. The region between OCMC and DDR is
  ** configured as device memory, with R/W access in user/privileged modes.
  ** Also, the region is marked 'Execute Never'.
  */
  REGION regionDev = {
                      MMU_PGTYPE_SECTION, START_ADDR_DEV, NUM_SECTIONS_DEV,
                      MMU_MEMTYPE_STRONG_ORD_SHAREABLE,
                      MMU_REGION_NON_SECURE,
                      MMU_AP_PRV_RW_USR_RW  | MMU_SECTION_EXEC_NEVER,
                      (unsigned int*)pageTable
                     };

  /* Initialize the page table and MMU */
  MMUInit((unsigned int*)pageTable);

  /* Map the defined regions */
  MMUMemRegionMap(&regionDdr);
  MMUMemRegionMap(&regionOcmc);
  MMUMemRegionMap(&regionDev);

  /* Now Safe to enable MMU */
  MMUEnable((unsigned int*)pageTable);
} /* MMUConfigAndEnable */

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  __early_init                                                         */
/*                                                                       */
/*  This function will be called after stack setup and before main.      */
/*                                                                       */
/*  Here the Clock and MMU/Cache can be setup for example. The SysTick   */
/*  will be setup later by the hal_lld_init.                             */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void __early_init (void)
{
  /*
   * Setup clock
   */
   
   /* The clock is still setup by the CrossWorks for ARM init. */
   

   /*
    * MMU and Cache is still setup by the CrossWorks for ARM
    * startup code, Sitara_Startup.s.
    */
#if 0   
  /*
   * Setup MMU and Cache
   */   
   
  /* Disable cache first */
  CacheDisable(CACHE_ALL);

  /* Setup and enable MMU */
  MMUConfigAndEnable();
    
  /* Enable the cache now */
  CacheEnable(CACHE_ALL);
#endif  

  /* Initialize the ARM interrupt control */
  IntAINTCInit();

} /* __early_init */

/*************************************************************************/
/*  boardInit                                                            */
/*                                                                       */
/*  This function will be called at the end of halInit.                  */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void boardInit (void) 
{
  uint32_t mode;
  
  /*
   * Init the 4 user LEDs
   */
  mode = (PAL_MODE_DIR_OUT   | PAL_MODE_SR_SLOW | PAL_MODE_RX_DISABLE |
          PAL_MODE_PULL_NONE | PAL_MODE_7 );
  
  palClearPad(IOPORT1, 21);
  palSetPadMode(IOPORT1, 21, mode); 
  
  palClearPad(IOPORT1, 22);
  palSetPadMode(IOPORT1, 22, mode); 
                           
  palClearPad(IOPORT1, 23);
  palSetPadMode(IOPORT1, 23, mode); 
                           
  palClearPad(IOPORT1, 24);
  palSetPadMode(IOPORT1, 24, mode); 
  
} /* boardInit */

/*** EOF ***/
