/**************************************************************************
*  Copyright (c) 2013 by Michael Fischer. All rights reserved.
*
*  Based on a template from ChibiOS/RT. Therefore partial copyright:
*  Copyright (c) 2006-2012, Giovanni Di Sirio
*
*  Based on an StarterWare example from TI. Therefore partial copyright:
*  Copyright (c) 2010 Texas Instruments Incorporated
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  18.02.2013 mifi  First Version for the BeageBone board.
*                   CrossWorks for ARM v2.3.0 was used.
**************************************************************************/

/*=======================================================================*/
/*  Include                                                              */
/*=======================================================================*/

#include "ch.h"
#include "hal.h"

#if HAL_USE_SERIAL || defined(__DOXYGEN__)

#include "interrupt.h"
#include "uart_irda_cir.h"

#include "hw_control_AM335x.h"
#include "soc_AM335x.h"
#include "hw_cm_wkup.h"
#include "hw_cm_per.h"
#include "hw_types.h"

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

#define UART_MODULE_INPUT_CLK   (48000000)

/*=======================================================================*/
/*  Prototypes                                                           */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all global Data                                        */
/*=======================================================================*/

/* UART0 serial driver identifier. */
#if (AM335X_SERIAL_USE_UART0 == TRUE)
SerialDriver SD0;
#endif

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

/*
 * Driver default configuration.
 */
static const SerialConfig default_config = 
{
  SERIAL_DEFAULT_BITRATE, SD_WORD_LENGTH_8, SD_PARITY_NONE, SD_NUM_STB_1
};


/*
 * Input / output buffer
 */
#if (AM335X_SERIAL_USE_UART0 == TRUE)
static uint8_t SD0InputBuffer[AM335X_SERIAL_UART0_BUFFER_SIZE];
static uint8_t SD0OutputBuffer[AM335X_SERIAL_UART0_BUFFER_SIZE];
#endif

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

#if (AM335X_SERIAL_USE_UART0 == TRUE)
/*************************************************************************/
/*  UART0ModuleClkConfig                                                 */
/*                                                                       */
/*  This function enables the system L3 and system L4_WKUP clocks.       */
/*  This also enables the clocks for UART0 instance.                     */
/*  (Based on the TI platform\baglebone\uart example)                    */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void UART0ModuleClkConfig (void)
{

  /* Writing to MODULEMODE field of CM_WKUP_UART0_CLKCTRL register. */
  HWREG(SOC_CM_WKUP_REGS + CM_WKUP_UART0_CLKCTRL) |=
        CM_WKUP_UART0_CLKCTRL_MODULEMODE_ENABLE;

  /* Waiting for MODULEMODE field to reflect the written value. */
  while(CM_WKUP_UART0_CLKCTRL_MODULEMODE_ENABLE !=
        (HWREG(SOC_CM_WKUP_REGS + CM_WKUP_UART0_CLKCTRL) &
         CM_WKUP_UART0_CLKCTRL_MODULEMODE));

  /*
  ** Waiting for CLKACTIVITY_UART0_GFCLK field in CM_WKUP_CLKSTCTRL
  ** register to attain desired value.
  */
  while(CM_WKUP_CLKSTCTRL_CLKACTIVITY_UART0_GFCLK !=
        (HWREG(SOC_CM_WKUP_REGS + CM_WKUP_CLKSTCTRL) &
         CM_WKUP_CLKSTCTRL_CLKACTIVITY_UART0_GFCLK));

  /*
  ** Waiting for IDLEST field in CM_WKUP_UART0_CLKCTRL register to attain
  ** desired value.
  */
  while((CM_WKUP_UART0_CLKCTRL_IDLEST_FUNC <<
         CM_WKUP_UART0_CLKCTRL_IDLEST_SHIFT) !=
        (HWREG(SOC_CM_WKUP_REGS + CM_WKUP_UART0_CLKCTRL) &
         CM_WKUP_UART0_CLKCTRL_IDLEST));
         
} /* UART0ModuleClkConfig */
#endif /* (AM335X_SERIAL_USE_UART0 == TRUE) */

/*************************************************************************/
/*  sd_lld_serve_interrupt                                               */
/*                                                                       */
/*  Common interrupt handler.                                            */
/*                                                                       */
/*  sdp     communication channel associated to the UART                 */
/*                                                                       */
/*  In    : sdp                                                          */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void sd_lld_serve_interrupt (SerialDriver *sdp) 
{
  uint32_t Base = sdp->base;
  uint32_t IntId;
  
  /* Checking ths source of UART interrupt. */
  IntId = UARTIntIdentityGet(Base);

  switch (IntId)
  {
    case UART_INTID_TX_THRES_REACH:
    {
      msg_t b;

      chSysLockFromIsr();
      b = chOQGetI(&sdp->oqueue);
      if (b < Q_OK) 
      {
        chIOAddFlagsI(sdp, IO_OUTPUT_EMPTY);

        /* Disabling the THR interrupt. */        
        UARTIntDisable(Base, UART_INT_THR);        
      }
      else
      {
        UARTCharPut(Base, b);      
      }  
      chSysUnlockFromIsr();
      break;
    } /* UART_INTID_TX_THRES_REACH */
    
    case UART_INTID_RX_THRES_REACH:
    {
      chSysLockFromIsr();
      sdIncomingDataI(sdp, UARTCharGetNonBlocking(Base));
      chSysUnlockFromIsr();
      break;
    } /* UART_INTID_RX_THRES_REACH */  

    case UART_INTID_RX_LINE_STAT_ERROR:
    {
      uint8_t rxByte;
      
      rxByte = UARTCharGetNonBlocking(Base);
      break;
    } /* UART_INTID_RX_LINE_STAT_ERROR */
    
    case UART_INTID_CHAR_TIMEOUT:
    {
      uint8_t rxByte;
      
      rxByte = UARTCharGetNonBlocking(Base);
      break;
    } /* UART_INTID_CHAR_TIMEOUT */  
  
    default:
    {
      /* Do nothing */
      break;
    }
  
  } /* end switch(intId) */
  
} /* sd_lld_serve_interrupt */

#if (AM335X_SERIAL_USE_UART0 == TRUE)
/*************************************************************************/
/*  UART0IrqHandler                                                      */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
CH_IRQ_HANDLER (UART0IrqHandler)
{
  CH_IRQ_PROLOGUE();
  sd_lld_serve_interrupt(&SD0);
  CH_IRQ_EPILOGUE();
} /* UART0IrqHandler */

/*************************************************************************/
/*  Notify0                                                              */
/*                                                                       */
/*  The UART notify handler, will be called if new data is available     */
/*  in the out queue.                                                    */
/*                                                                       */
/*  qp      the queue which is used for the UART                         */
/*                                                                       */
/*  In    : qp                                                           */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void Notify0 (GenericQueue *qp) 
{
  (void)qp;
  
  /* Enable TX interrupt */
  UARTIntEnable(SOC_UART_0_REGS, UART_INT_THR);
} /* Notify0 */
#endif /* (AM335X_SERIAL_USE_UART0 == TRUE) */

/*************************************************************************/
/*  uart_deinit                                                          */
/*                                                                       */
/*  UART de-initialization.                                              */
/*                                                                       */
/*  sdp     communication channel associated to the UART                 */
/*                                                                       */
/*  In    : sdp                                                          */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void uart_deinit (SerialDriver *sdp)
{
  uint32_t Base = sdp->base;
  
  /* Disable all interrupts of the UART */
  UARTIntDisable(Base, -1);
  
  /* 
   * Disable system interrupt 
   */
  switch (Base)
  {
  
#if (AM335X_SERIAL_USE_UART0 == TRUE)
    case SOC_UART_0_REGS:
    {
      /* Disable the system interrupt in AINTC. */
      IntSystemDisable(SYS_INT_UART0INT);    
      break;
    } /* SOC_UART_0_REGS */
#endif    
    
    default:
    {
      /* Do nothing */
      break;
    }
  } /* end switch (Base) */
  
} /* uart_deinit */

/*************************************************************************/
/*  uart_init                                                            */
/*                                                                       */
/*  UART initialization.                                                 */
/*                                                                       */
/*  sdp     communication channel associated to the UART                 */
/*  config  the architecture-dependent serial driver configuration       */
/*                                                                       */
/*  In    : sdp, config                                                  */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void uart_init (SerialDriver *sdp, const SerialConfig *config) 
{
  uint32_t Base = sdp->base;
  uint32_t FifoConfig;
  uint32_t DivisorValue; 
  
  /* Module reset. */
  UARTModuleReset(Base);
  
  
  /*
   * FIFO configurations
   */
  
  /* Setting the TX and RX FIFO Trigger levels as 1. No DMA enabled. */
  FifoConfig = UART_FIFO_CONFIG(UART_TRIG_LVL_GRANULARITY_1,
                                UART_TRIG_LVL_GRANULARITY_1,
                                1,
                                1,
                                1,
                                1,
                                UART_DMA_EN_PATH_SCR,
                                UART_DMA_MODE_0_ENABLE);

  /* Configuring the FIFO settings. */
  UARTFIFOConfig(Base, FifoConfig);


  /*
   * Baud Rate settings
   */

  /* Computing the Divisor Value. */
  DivisorValue = UARTDivisorValCompute(UART_MODULE_INPUT_CLK,
                                       config->BaudRate,
                                       UART16x_OPER_MODE,
                                       UART_MIR_OVERSAMPLING_RATE_42);

  /* Programming the Divisor Latches. */
  UARTDivisorLatchWrite(Base, DivisorValue);

  /* Switching to Configuration Mode B. */
  UARTRegConfigModeEnable(Base, UART_REG_CONFIG_MODE_B);
  
  /* Programming the Line Characteristics. */
  UARTLineCharacConfig(Base, 
                       (config->WordLength | config->NumStb), 
                       config->Parity);

  /* Disabling write access to Divisor Latches. */
  UARTDivisorLatchDisable(Base);

  /* Disabling Break Control. */
  UARTBreakCtl(Base, UART_BREAK_COND_DISABLE);

  /* Switching to UART16x operating mode. */
  UARTOperatingModeSelect(Base, UART16x_OPER_MODE);


  /* 
   * Enable system interrupt 
   */
  switch (Base)
  {
  
#if (AM335X_SERIAL_USE_UART0 == TRUE)
    case SOC_UART_0_REGS:
    {
      /* Registering the Interrupt Service Routine(ISR). */
      IntRegister(SYS_INT_UART0INT, UART0IrqHandler);

      /* Setting the priority for the system interrupt in AINTC. */
      IntPrioritySet(SYS_INT_UART0INT, 
                     AM335X_SERIAL_UART0_PRIORITY, AINTC_HOSTINT_ROUTE_IRQ);

      /* Enabling the system interrupt in AINTC. */
      IntSystemEnable(SYS_INT_UART0INT);    
      break;
    } /* SOC_UART_0_REGS */
#endif
    
    default:
    {
      /* Do nothing */
      break;
    }
  } /* end switch (Base) */
    
  /* Enabling the specified UART interrupts. */
  UARTIntEnable(Base, (UART_INT_LINE_STAT | UART_INT_RHR_CTI));
    
} /* uart_init */

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  sd_lld_init                                                          */
/*                                                                       */
/*  Low level serial driver initialization.                              */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void sd_lld_init (void) 
{
#if (AM335X_SERIAL_USE_UART0 == TRUE)
  /*
   * UART0 initialization
   */
  SD0.ib   = SD0InputBuffer;
  SD0.ob   = SD0OutputBuffer;
  SD0.size = AM335X_SERIAL_UART0_BUFFER_SIZE;
  sdObjectInit(&SD0, NULL, Notify0);
  SD0.base = SOC_UART_0_REGS;

  /* Configuring the system clocks. */
  UART0ModuleClkConfig();
  
  /* 
   * Performing Pin Multiplexing. 
   */
   
  /* GPIO1_10 RXD */
  palSetPadMode(IOPORT1, 10, (PAL_MODE_RX_ENABLE | PAL_MODE_PULL_UP | PAL_MODE_0));

  /* GPIO1_11 TXD */
  palSetPadMode(IOPORT1, 11, (PAL_MODE_PULL_UP | PAL_MODE_0));
#endif /* (AM335X_SERIAL_USE_UART0 == TRUE) */ 

} /* sd_lld_init */

/*************************************************************************/
/*  sd_lld_start                                                         */
/*                                                                       */
/*  Low level serial driver configuration and (re)start.                 */
/*                                                                       */
/*  sdp       pointer to a SerialDriver object                           */
/*  config    the architecture-dependent serial driver configuration.    */
/*            If this parameter is set to NULL then a default            */
/*            configuration is used.                                     */
/*                                                                       */
/*  In    : sdp, config                                                  */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void sd_lld_start (SerialDriver *sdp, const SerialConfig *config)
{
  if (config == NULL)
  {
    config = &default_config;
  }    

  if (sdp->state == SD_STOP) 
  {
    uart_init(sdp, config);
  }
} /* sd_lld_start */

/*************************************************************************/
/*  sd_lld_stop                                                          */
/*                                                                       */
/*  Low level serial driver stop.                                        */
/*                                                                       */
/*  sdp       pointer to a SerialDriver object                           */
/*                                                                       */
/*  In    : sdp                                                          */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void sd_lld_stop (SerialDriver *sdp)
{
  if (sdp->state == SD_READY)
  {
    uart_deinit(sdp);
  }    
} /* sd_lld_stop */

#endif /* HAL_USE_SERIAL */
/*** EOF ***/
